package it.unitn.disi.MR;

import java.util.Random;
import javafx.animation.ParallelTransition;
import javafx.animation.TranslateTransition;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.Pane;
import javafx.stage.Stage;

/**
 * Finestra principale - funge anche da controller per l'applicazione
 */
public class MainWindow extends BorderPane {

    Griglia griglia;
    boolean loopIsActive=false;
    private int availableCars = 3;
    private Button addCarButton = null;
    private final Label availableCarsLabel = new Label();
    private ControlWindow controlWindow = new ControlWindow(this);
    private Random randomGenerator = new Random(System.currentTimeMillis());
    private boolean loop;
    private double baseX = 0;
    private double baseY = 0;

    public MainWindow() {
        griglia = new Griglia(this);
        this.setCenter(griglia);
        //==
        Pane bottomPane = new FlowPane();
        addCarButton = new Button() {
            {   // questo blocco gioca il ruolo del costruttore 
                //per la classe interna anonima
                setText("Aggiungi auto");
                addEventHandler(ActionEvent.ACTION, new EventHandler<ActionEvent>() {
                    public void handle(ActionEvent event) {
                        Strada.addCarMode = true;
                    }
                });
            }
        };
        availableCarsLabel.setText("Auto disponibili " + availableCars);
        bottomPane.getChildren().add(availableCarsLabel);
        bottomPane.getChildren().add(addCarButton);
        bottomPane.getChildren().add(new Button() {
            {   // questo blocco gioca il ruolo del costruttore 
                //per la classe interna anonima
                setText("Inizia");
                addEventHandler(ActionEvent.ACTION, new EventHandler<ActionEvent>() {
                    public void handle(ActionEvent event) {
                        controlWindow.setX(baseX);
                        controlWindow.setY(baseY);
                        controlWindow.show();
                    }
                });
            }
        });
        this.setBottom(bottomPane);
    }

    void setBaseX(double x) {
        baseX = x;
    }

    void setBaseY(double y) {
        baseY = y;
    }

    void add(Node n, int i, int j) {
        griglia.add(n, i, j);
    }
    
    boolean remove(Node n) {
        return griglia.getChildren().remove(n);
    }
    /**
     * diminuisci le macchine disponibili e controlla il relativo bottone
     */
    void decreaseAvailableCars() {
        availableCars--;
        availableCarsLabel.setText("Auto disponibili " + availableCars);
        if (availableCars == 0) {
            addCarButton.setDisable(true);
        }
    }
    /**
     * muovi le macchine - Se dx e dy sono uguali, movimento random personalizzato
     * per ciascuna macchina
     *
     * @param dx spostamento sulle x: -1, 0 o 1
     * @param dy spostamento sulle y: --1, 0 o 1
     */
    void moveCars(int dx, int dy) {
        ParallelTransition pt = new ParallelTransition();
        for (Car c : Car.carList) {
            Strada destination=null;
            if (dx != dy) {
                destination = c.checkCarMove(dx, dy);
            } else {
                switch (randomGenerator.nextInt(4)) {
                    case 0:
                        destination = c.checkCarMove(1, 0);
                        break;
                    case 1:
                        destination = c.checkCarMove(-1, 0);
                        break;
                    case 2:
                        destination = c.checkCarMove(0, 1);
                        break;
                    case 3:
                        destination = c.checkCarMove(0, -1);
                        break;
                }
            }
            if (destination != null) {
                TranslateTransition tt=c.prepareCarTranslation(destination);
                pt.getChildren().add(tt);
                System.out.println("adding prepared transition " + tt);
            }
        }
        // alla fine di tutte le transizione (eseguite in parallelo)
        // aggiorna le posizioni finali delle auto 
        // e verifica eventuali collisioni
        pt.setOnFinished(new EventHandler<ActionEvent>() {
            public void handle(ActionEvent event) {
                finalizeCarMovements();
                // se siamo in un loop, eseguiamo la prossima mossa
                if (loopIsActive) {
                    moveCars(0,0);
                }
            }
        });
        pt.play();
    }
    // aggiorna le posizioni finali delle auto e verifica eventuali collisioni
    private void finalizeCarMovements() {
        // cerca collisioni
        for (Car c : Car.carList) {
            for (Car c1 : Car.carList) {
                if (c == c1) {
                    continue;
                }
                if (c.getDestination() == c1.getDestination()) {
                    crash();
                    return;
                }
            }
        }
        //per le auto che si sono spostate, rimuovi auto dalla posizione di partenza 
        for (Car c : Car.carList) {
            if (c.getCurrentPosition() != c.getDestination()) 
                c.getCurrentPosition().removeCar();
        }
        //per le auto che si sono spostate, aggiungi auto dalla posizione di arrivo 
        for (Car c : Car.carList) {
            Strada destination=c.getDestination();
            if (c.getCurrentPosition() != destination) {
                destination.addCar(c);
                c.setCurrentPosition(destination);
            }
        }
    }

    private void crash() {
        this.loopIsActive=false;
        System.out.println("two cars collided!");
        Stage s=(Stage)(this.getScene().getWindow());
        ModalDialog m=new ModalDialog();
        m.showPopup("Crash", s);
    } 
}
